/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : BatteryDischarge.c
* Version : 1.0
* Device(s) : R7F0C809
* H/W Platform : YCB14R7F0C809001B
* Description : This file implements device driver for initializes system.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.08.2014 1.00 First Release
******************************************************************************/

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "BatteryDischarge.h"
#include "BatteryCharge.h"

/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
uint16_t g_Battery_AllowDischargeFlag=0;					/* Allow discharge flag */
uint16_t g_Load_ADVlaue = 0;								/* Record battery voltage */
uint16_t Load_CheckDelay = 0;								/* Load check count */
uint16_t g_Load_CheckCount = 0;								/* Load Lost count */
uint16_t Battery_DischargeFlag = 0;							/* Battery discharge  flag */
	
/******************************************************************************
* Function Name : Load_Scan
* Description : This function is scan load.
* Arguments : none
* Return Value : none	
******************************************************************************/	
void Load_Scan(void)		
{	
	if(g_Battery_AllowDischargeFlag == 1)					/* Battery discharging? */	
	{
		Load_CheckDelay++;									/* Load check time count */
		if(Load_CheckDelay == 100)							/* Load checkl time == 2s */
		{
			Battery_PowerScan();							/* Battery voltage detection */
			if(g_Load_ADVlaue > g_AD_value)					/* Battery voltage > Record voltage */
			{
				if((g_Load_ADVlaue - g_AD_value)  > 0x200)	/* Voltage difference>0.2V */
				{
					Battery_DischargeFlag = 1;				/* continue discharge flag set 1 */
					g_Load_CheckCount = 0;					/* Interval scan time clear */
					g_Load_ADVlaue = g_AD_value;			/* Record battery voltage */
				}	
			}
			else if( g_Load_ADVlaue < g_AD_value )			/* Battery voltage > Record voltage */
			{
				if((g_AD_value - g_Load_ADVlaue)  > 0x200)	/* Voltage difference>0.2V */
				{
					g_Load_CheckCount++;					/* Load lost time count */
					if(g_Load_CheckCount == 10)				/* Lost time == 20s */
					{
						Battery_DischargeOFF();				/* Discharge off */
						g_Load_ADVlaue = 0;					/* Record voltage clear */
						Battery_DischargeFlag = 0;			/* Discharging flag clear */
						g_Load_CheckCount = 0;				/* Lost time clear */
						g_Battery_AllowDischargeFlag = 0;	/* Allow discharge flag clear */
					}
				}
				else
				{
					g_Load_CheckCount = 0;					/* Lost time clear */
				}
			}
			if(Battery_DischargeFlag == 0)					/* Discharging flag == 0 */
			{	
				g_Load_CheckCount++;						/* Load lost time count */
				if(g_Load_CheckCount == 10)					/* Lost time == 20s */
				{		
					Battery_DischargeOFF();					/* Discharge off */
					g_Load_ADVlaue = 0;						/* Record voltage clear */
					Battery_DischargeFlag = 0;				/* Discharging flag clear */
					g_Load_CheckCount = 0;					/* Lost time clear */
					g_Battery_AllowDischargeFlag = 0;		/* Allow discharge flag clear */
				}
			}
			if(g_Load_CheckCount < 1)						/* Load lost time count <1 */
			{
			   	g_Load_ADVlaue = g_AD_value;				/* Record battery voltage */
			}
			Load_CheckDelay = 0;							/* Load check time clear */
		}
	}	
}
/******************************************************************************
End of function Load_Scan
******************************************************************************/

/******************************************************************************
* Function Name : Battery_DischargeON
* Description : This function is open the discharge.
* Arguments : none
* Return Value : none	
******************************************************************************/	
void Battery_DischargeON(void)
{
	P1 |= 0x01;												/* Discharge on */
}
/******************************************************************************
End of function Battery_DischargeON
******************************************************************************/

/******************************************************************************
* Function Name : Battery_DischargeOFF
* Description : This function is close the discharge.
* Arguments : none
* Return Value : none	
******************************************************************************/	
void Battery_DischargeOFF(void)
{
	P1 &= 0xfe;												/* Discharge off */
}
/******************************************************************************
End of function Battery_DischargeOFF
******************************************************************************/