/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING 
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT 
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR 
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE 
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software 
* and to discontinue the availability of this software.  By using this software, 
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : adc.c
* Version      : 1.0
* Device(s)    : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description  : This file implements device driver for ADC module.
***********************************************************************************************************************/

/***********************************************************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.09.2014 1.00 First Release
***********************************************************************************************************************/

/*******************************************************************************
** Pragma directive
*******************************************************************************/
#pragma interrupt INTAD INTAD_interrupt

/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes <System Includes> , <Project Includes>
***********************************************************************************************************************/
#include "userdefine.h"
#include "adc.h"
#include "monitor.h"

/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
extern uint8_t g_SensorSimpleTimes;
extern uint8_t g_fSensorComputeReq;
extern uint8_t g_fSensorSimpling;
extern uint16_t g_aSensorAdValue[];

uint16_t ad_value[AD_MAX_SAMPLE_TIMES];

/***********************************************************************************************************************
* Function Name: ADC_Init
* Description  : This function initializes the AD converter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void ADC_Init(void)
{
	ADCEN = 1U;			/* supply AD clock */
    	ADM0 = 0x00U;			/* disable AD conversion and clear ADM0 register */
    	ADMK = 1U;			/* disable INTAD interrupt */
    	ADIF = 0U;			/* clear INTAD interrupt Flag */
    	/* Set INTAD level1 priority */
    	ADPR1 = 0U;
    	ADPR0 = 1U;
    
    	ADM0 = 0x28U;			/* conversion time mode select normal1 mode, conversion clock select fclk/5,conversion time = 3.96us */
    	ADM1 = 0x20U;			/* Software trigger mode,One-shot conversion mode */
    	ADM2 = 0x00U;			/* Supplied from VDD and VSS,select 10-bit resolution */
	
	ADMK = 1U;			/* disable INTAD interrupt */
}
/***********************************************************************************************************************
End of function ADC_Init
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: ADC_Start
* Description  : This function starts the AD converter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void ADC_Start(void)
{
	uint8_t i = 0;	
	while(1==ADCS);
    	ADIF = 0U;			/* clear INTAD interrupt flag */
	ADMK = 0U;			/* enable INTAD interrupt */
    	ADCE = 1U;			/* enable AD comparator */
    	for(i = 0; i<25; i++);		/* wait 1.0us */
  	ADCS = 1U;			/* enable AD conversion */
}
/***********************************************************************************************************************
End of function ADC_Start
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: ADC_Stop
* Description  : This function stops the AD converter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void ADC_Stop(void)
{
	ADCS = 0U;			/* disable AD conversion */
    	ADCE = 0U;			/* disable AD comparator */
    	ADMK = 1U;			/* disable INTAD interrupt */
    	ADIF = 0U;			/* clear INTAD interrupt flag */
}
/***********************************************************************************************************************
End of function ADC_Stop
***********************************************************************************************************************/


/***********************************************************************************************************************
* Function Name: ADC_Sample
* Description  : This function starts the AD converter.
* Arguments    : None
* Return Value : None 
***********************************************************************************************************************/
uint16_t ADC_Sample(uint8_t channel,uint8_t times)
{
	uint16_t i,j,min,tmp;	
	uint16_t result;
	DI();
	ADS = channel;
	while(1==ADCS);
    	ADIF = 0U;			/* clear INTAD interrupt flag */
	ADMK = 0U;			/* enable INTAD interrupt */
    	ADCE = 1U;			/* enable AD comparator */
    	for(i = 0; i<25; i++);		/* wait 1.0us */
	for(i = 0;i<times;i++)
	{
  		ADCS = 1U;		/* enable AD conversion */
		while (0U == ADIF);
    		ADIF = 0U;
		ad_value[i] = (ADCR >> 6U);
	}
	/* Sort the array */
	for(i=0;i<times;i++)
	{
		min = ad_value[i];
		for(j=i;j<times;j++)
		{
			if(ad_value[j]<min)
			{
				tmp = min;
				min = ad_value[j];
				ad_value[j] = tmp;
			}
		}
		ad_value[i] = min;
	}
	/*Get average value*/
	tmp = 0;
	for(i=0;i<times-2;i++)
	{
		tmp += ad_value[i+1];
	}
	result = tmp/(times-2);
	return result;
	EI();
}
/***********************************************************************************************************************
End of function ADC_Sample
***********************************************************************************************************************/



/******************************************************************************
* Function Name: INTAD_interrupt
* Description : This function is INTAD interrupt service routine.
* Arguments : none
* Return Value : none
******************************************************************************/
__interrupt void INTAD_interrupt(void)
{
	g_aSensorAdValue[g_SensorSimpleTimes++] = (ADCR >> 6U);
	ADC_Stop();
	if(g_SensorSimpleTimes>=SENSOR_SIMPLE_TIMES)
	{
		g_SensorSimpleTimes = 0;
		g_fSensorComputeReq = 1;
		TT0 |= 0x0003;		/* Stops TAU00~TAU01 */
		g_fSensorSimpling = OFF;
	}
		
}
/******************************************************************************
End of function INTAD_interrupt
******************************************************************************/