/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : beep.c
* Version : 1.0
* Device(s) : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description : This file implements buzzer function.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.09.2015 1.00 First Release
******************************************************************************/

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "beep.h"

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

uint8_t g_BeepCount = 0;	/* Record the times of beep sound */
uint8_t g_BeepKeepTime = 0; 	/* Record the time of beep tone on */
uint8_t g_fBeepOnReq =0;    	/* Beep tone on request flag */
uint8_t g_fAlarmReq = 0;	/* Alarm on request flag */

/******************************************************************************
* Function Name:Beep_Times_Set
* Description : This function implements setting beep times function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Beep_Times_Set(uint8_t times)
{
    /* Times of beep tone set */
    g_BeepCount = times * 2;
    g_BeepKeepTime = BEEP_ON_TIME;
    g_fBeepOnReq = ON;
    BEEP = ON;
}
/******************************************************************************
End of set beep times function
******************************************************************************/

/******************************************************************************
* Function Name:Alarm_On
* Description : This function implements alarm on function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Alarm_On(void)
{
    /* Alarm on set */
    g_fAlarmReq = ON;
    g_BeepKeepTime = BEEP_ON_TIME;
    g_fBeepOnReq = ON;
    BEEP = ON;
}
/******************************************************************************
End of Alarm_On function
******************************************************************************/

/******************************************************************************
* Function Name:Alarm_Off
* Description : This function implements alarm off function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Alarm_Off(void)
{
    /* Alarm off set */
	g_fAlarmReq = OFF;
    	g_BeepKeepTime = 0;
    	g_fBeepOnReq = OFF;
    	BEEP = OFF;
}
/******************************************************************************
End of Alarm_Off function
******************************************************************************/

/******************************************************************************
* Function Name:Beep_Process
* Description : This function implements beep process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Beep_Process(void)
{
	/* Buzzer control */
    	if(g_fAlarmReq == ON)
    	{
		g_BeepKeepTime--;
        	if(g_BeepKeepTime <= 0)
        	{
			g_BeepKeepTime = BEEP_ON_TIME;
			g_fBeepOnReq = 1 - g_fBeepOnReq;
    		}
		BEEP = g_fBeepOnReq;
	}		
	if(g_BeepCount > 0 )
    	{
        	g_BeepKeepTime--;
        	if(g_BeepKeepTime <= 0)
        	{
            		g_fBeepOnReq = 1 - g_fBeepOnReq;
           		g_BeepCount--;
            		if(g_BeepCount <= 0)
            		{
               			g_BeepKeepTime = 0;
                		g_fBeepOnReq = OFF;
            		}
            		else
            		{
                		g_BeepKeepTime = BEEP_ON_TIME;
            		}
           	 	BEEP = g_fBeepOnReq;
        	}   
	}
}
/******************************************************************************
End of beep process function
******************************************************************************/