/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name    : lcd.c
* Version      : 1.0
* Device(s)    : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description  : This file implements device driver for LCD module.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 20.09.2015 1.00 First Release
******************************************************************************/

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "lcd.h"
#include "battery.h"
#include "string.h"

/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
uint16_t g_DisplayData[4] = {0,0,0,0};                                   /* Variable for data display */
uint16_t g_NumberData[14] = {0x0e0b,0x0600,0x0c07,0x0e05,0x060c,0x0a0d, /* "0'~"9","P","C" ,"L" and NULL */
                             0x0a0f,0x0e00,0x0e0f,0x0e0d,0x0c0e,0x080b,0x000b,0x0000};
			     
uint8_t g_fNewDataDispRq;		/* Data display area update request flag */
uint8_t g_fBatteryFlagDispRq;		/* Battery flag display request flag */

extern uint8_t g_ChargeStatus;
extern uint8_t g_fChargeConnected;
extern uint8_t g_MainMode;
extern uint16_t g_DustDensityValue;
extern uint16_t g_BatterVoltage;
extern uint8_t g_BatteryCapacityIndicator;
extern _DUST_ALARM_DATA g_sDustAlarmTempData;
extern uint8_t g_DustAlarmDataSetStep;
extern uint8_t g_fAlarmDataSetFlashOn;
extern uint8_t g_fAlarmOnFlashOn;

static void Display_Initial_Date(void);
static void Display_Dust_Density(uint16_t density);
static void Display_Alarm_Value_Set(uint8_t display);
static void Display_Battery_Indicator(uint8_t step);
static void Display_Battery_Capacity_Percent(uint16_t voltage);
static void Display_Empty_Date(void);

/******************************************************************************
* Function Name : Display_Initial_Date
* Description   : This function makes display content of initial mode.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
static void Display_Initial_Date(void)
{
	/* Display content of initial mode */
	SEG0 = 0x0f;
	SEG1 = 0x0e;
	SEG2 = 0x0f;
	SEG3 = 0x0f;
	SEG4 = 0x0f;
	SEG5 = 0x0e;
	SEG6 = 0x0f;
	SEG7 = 0x0e;
	SEG24 = 0x0f;
	SEG25 = 0x04;
}
/******************************************************************************
End of function Display_Initial_Date
******************************************************************************/

/******************************************************************************
* Function Name : Display_Dust_Density
* Description   : This function makes the display content of dust density.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
static void Display_Dust_Density(uint16_t density)
{
	/* Display content of dust density */
	g_DisplayData[0] = DISP_P;
	g_DisplayData[1] = density/100;
	g_DisplayData[2] = (density - (g_DisplayData[1]*100))/10;
	g_DisplayData[3] = density - (g_DisplayData[1]*100) - (g_DisplayData[2]*10);
	memcpy(LCD_NUM1, &g_NumberData[g_DisplayData[0]], 2);    /* Displays Num1 */
        memcpy(LCD_NUM2, &g_NumberData[g_DisplayData[1]], 2);    /* Displays Num2 */
        memcpy(LCD_NUM3, &g_NumberData[g_DisplayData[2]], 2);    /* Displays Num3 */
        memcpy(LCD_NUM4, &g_NumberData[g_DisplayData[3]], 2);    /* Displays Num4 */
}
/******************************************************************************
End of function Display_Dust_Density
******************************************************************************/

/******************************************************************************
* Function Name : Display_Alarm_Value_Set
* Description   : This function controls the alarm value setting display.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
static void Display_Alarm_Value_Set(uint8_t display)
{
	/* Display content of alarm value setting */
	g_DisplayData[0] = DISP_P;
	g_DisplayData[1] = g_sDustAlarmTempData.High;
	g_DisplayData[2] = g_sDustAlarmTempData.Middle;
	g_DisplayData[3] = g_sDustAlarmTempData.Low;
	if(DISP_OFF == display)
	{
		g_DisplayData[g_DustAlarmDataSetStep+1] = DISP_NULL;
	}
	memcpy(LCD_NUM1, &g_NumberData[g_DisplayData[0]], 2);    /* Displays Num1 */
        memcpy(LCD_NUM2, &g_NumberData[g_DisplayData[1]], 2);    /* Displays Num2 */
        memcpy(LCD_NUM3, &g_NumberData[g_DisplayData[2]], 2);    /* Displays Num3 */
        memcpy(LCD_NUM4, &g_NumberData[g_DisplayData[3]], 2);    /* Displays Num4 */
}
/******************************************************************************
End of function Display_Dust_Density
******************************************************************************/

/******************************************************************************
* Function Name : Display_Dust_Density
* Description   : This function controls the battery indicator display.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
static void Display_Battery_Indicator(uint8_t step)
{
	/* Battery indicator display control */
	switch(step)
	{
		case 0:
			SEG24 = 0x0000;
			break;
		case 1:
			SEG24 = 0x0001;
			break;
		case 2:
			SEG24 = 0x0003;
			break;
		case 3:
			SEG24 = 0x0007;
			break;
		default:
			SEG24 = 0x000f;
	}
			
}
/******************************************************************************
End of function Display_Dust_Density
******************************************************************************/

/******************************************************************************
* Function Name : Display_Battery_Capacity_Percent
* Description   : This function makes the display content of battery capacity percent.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
static void Display_Battery_Capacity_Percent(uint16_t voltage)
{
	/* Display content of battery capacity percent */
	if(1U == g_fChargeConnected)
	{
		g_DisplayData[0] = DISP_C;
	}
	else
	{
		g_DisplayData[0] = DISP_L;
	}
	if(voltage >= AD_BAT_MAX_VALUE)
	{
		g_DisplayData[1] = 1;
		g_DisplayData[2] = 0;
		g_DisplayData[3] = 0;
	}
	else
	{
		g_DisplayData[1] = 0;
		g_DisplayData[2] = (((voltage-AD_BAT_MIN_VALUE)*100)/(AD_BAT_MAX_VALUE-AD_BAT_MIN_VALUE))/10;
		g_DisplayData[3] = (((voltage-AD_BAT_MIN_VALUE)*100)/(AD_BAT_MAX_VALUE-AD_BAT_MIN_VALUE))%10;		
	}
	memcpy(LCD_NUM1, &g_NumberData[g_DisplayData[0]], 2);    /* Displays Num1 */
        memcpy(LCD_NUM2, &g_NumberData[g_DisplayData[1]], 2);    /* Displays Num2 */
        memcpy(LCD_NUM3, &g_NumberData[g_DisplayData[2]], 2);    /* Displays Num3 */
        memcpy(LCD_NUM4, &g_NumberData[g_DisplayData[3]], 2);    /* Displays Num4 */
	SEG3 |= 0x01; 						/*display dot*/
	
}
/******************************************************************************
End of function Display_Battery_Capacity_Percent
******************************************************************************/

/******************************************************************************
* Function Name : Display_Empty_Date
* Description   : This function implements clearing the LCD.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
static void Display_Empty_Date(void)
{
	/* Data display area clear */
	SEG0 = 0x00;
	SEG1 = 0x00;
	SEG2 = 0x00;
	SEG3 = 0x00;
	SEG4 = 0x00;
	SEG5 = 0x00;
	SEG6 = 0x00;
	SEG7 = 0x00;
	SEG24 = 0x00;
	SEG25 = 0x00;
}
/******************************************************************************
End of function Display_Empty_Date
******************************************************************************/
/******************************************************************************
* Function Name : LCD_Init
* Description   : This function implements LCD initialization process.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void LCD_Init(void)
{
    uint16_t w_count;
    RTCEN = 1;       /* Supply LCD clock */
    LCDON = 0;       /* Disable LCD clock operation */
    LCDM1 |= 0x00;
    LCDM0 |= 0x4d;   /* 4COM, 1/3 BIAS */

    ISCLCD &= 0x02;  /* Set CAPL and CAPH pins */
    PU12.7=0;
    P12.7=0;
    PU12.6=0;
    P12.6=0;

    PFSEG0 = 0xf0;   /* Set segment pins (SEG4~SEG7) */
    PM1 &= 0x1f;     /* P15~P17(SEG4~SEG6) */
    P1 &= 0x1f;
    PM5.0 = 0;       /* P50(SEG7) */
    P5.0 = 0;
    
    PFSEG3 = 0x03;   /* Set segment pins (SEG24~SEG25) */
    PMC4 = 0xfd;
    PM4.1 = 0;
    P4.1 = 0;
    
    PMC12 = 0xfe;
    PM12.0 = 0;
    P12.0 = 0;

    LCDC0 = 0x06;    /* fSUB/2^7 or fIL/2^7 */
    VLCD = 0x04;     /* 1.00 V (1/3 bias: 3.00 V) */
    /* Delay */
    for (w_count = 0; w_count <= 100; w_count++)
    {
        NOP();
    }
    VLCON = 1;       /* Enables voltage boost circuit or capacitor split circuit operation */
    /* Delay */
    for (w_count = 0; w_count <= 100; w_count++)
    {
        NOP();
    }
    SCOC = 1;        /* For display on */
}
/******************************************************************************
End of function LCD_Init
******************************************************************************/

/******************************************************************************
* Function Name: LCD_Start
* Description  : This function enables the LCD display.
* Arguments    : None
* Return Value : None
******************************************************************************/
void LCD_Start(void)
{
    LCDON = 1;    /* Display on */
}
/******************************************************************************
End of function LCD_Start
******************************************************************************/

/******************************************************************************
* Function Name: LCD_Stop
* Description  : This function disables the LCD display.
* Arguments    : None
* Return Value : None
******************************************************************************/
void LCD_Stop(void)
{
    LCDON = 0;   /* Display off */
}
/******************************************************************************
End of function LCD_Stop
******************************************************************************/

/******************************************************************************
* Function Name : Display_Process
* Description   : This function implements display process.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
void Display_Process(void)
{
	/*"-" flag display*/
	if(1U == g_fAlarmOnFlashOn)
	{
		SEG25 |= 0x04;
	}
	else
	{
		SEG25 &= 0xfb;
	}
	/*battery flag display*/
	if(1U == g_fBatteryFlagDispRq)
	{
		g_fBatteryFlagDispRq = OFF;
		Display_Battery_Indicator(g_BatteryCapacityIndicator);
	}
	
	/*4 digital display area*/
	if(1U == g_fNewDataDispRq)
	{
		g_fNewDataDispRq = OFF;
		switch(g_MainMode)
    		{
			case INITIAL_MODE:
				Display_Initial_Date();
				break;
			case BATTERY_MODE:
				Display_Battery_Capacity_Percent(g_BatterVoltage);
				break;
			case NORMAL_MODE:
			case ALARM_STANDBY_MODE:
			case ALARM_ON_MODE:
				Display_Dust_Density(g_DustDensityValue);
				break;
			case ALARM_SETTING_MODE:
				Display_Alarm_Value_Set(g_fAlarmDataSetFlashOn);
				break;
			default:
				Display_Empty_Date();
				
		}
    	}
}
/******************************************************************************
End of function Display_Process
******************************************************************************/
