/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : mode.c
* Version : 1.0
* Device(s) : R7F0C809
* H/W Platform : YCB14R7F0C809200B
* Description : This file implements mode process function.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.03.2015 1.00 First Release
******************************************************************************/
#pragma "sfr"
#pragma EI
#pragma DI
#pragma NOP
/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "mode.h"
#include "define.h"

/******************************************************************************
Macro definitions
******************************************************************************/



/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
extern void Beep_Times_Set(uint8_t times);
extern void Display_Data_Set(uint8_t DispString[],uint16_t DispTime);
extern void Display_Dynamic_Data_Set(uint8_t DispString[],uint16_t DispTime, uint8_t counter);
extern void Display_Data_Clear(void);
extern void Input_Number_Clear(void);
extern void Display_Input_Numbers(void);
extern void Motor_On(void);
extern void Motor_Off(void);
extern uint16_t g_ModeTimeOutCount;
extern uint16_t g_DispTimeCount;
extern uint16_t g_MotorWorkTimeCount;
extern uint16_t g_DoorOpenTime;
extern uint8_t g_fDoorOpen;
extern uint8_t g_fKeyProcessReq;
extern uint8_t g_KeyScanCount;
extern uint8_t g_KeyInputCount;
extern uint8_t g_fKeyOffReq;
extern uint8_t g_KeyValue;

_MAIN_MODE g_MainMode = INITIAL_MODE;       /* main mode */
uint8_t g_SubMode = 0;                      /* sub mode */
uint8_t g_aInputNumber[COM_NUM];            /* Save input Numbers */
uint8_t g_PasswordInputErrorCount = 0;      /* Record the input times of wrong password*/
uint8_t g_fPassWordSaveReq = FALSE;         /* Password save request */
uint8_t g_aPassWord[COM_NUM] = {0};         /* Save password */
uint8_t g_aSuperPassWord[COM_NUM] = {0};    /* Save super password */
uint8_t g_aNewPassWord[COM_NUM] = {0};      /* Save new super password */

/* Display string */
const uint8_t g_aDispGood[COM_NUM] = {'-','V','0','0','1','-'};     /* Version information display at the begin*/
const uint8_t g_aDispError[COM_NUM] = {'E','r','r','o','r','-'};    
const uint8_t g_aDispHold15[COM_NUM] = {'H','O','L','D','1','5'};
const uint8_t g_aDispHold[COM_NUM] = {'H','O','L','D',0xff,0xff};
const uint8_t g_aDispProg[COM_NUM] = {'P','R','O','G',0xff,0xff};
const uint8_t g_aDispNew[COM_NUM] = {'N','E','W',0xff,0xff,0xff};
const uint8_t g_aDispAgain[COM_NUM] = {'A','G','A','I','N',0xff};
const uint8_t g_aDispDone[COM_NUM] = {'D','O','N','E',0xff,0xff};
const uint8_t g_aDispClosed[COM_NUM] = {'C','L','O','S','E','D'};
const uint8_t g_aDispOpened[COM_NUM] = {'O','P','E','N','E','D'};
const uint8_t g_aDispClose[COM_NUM] = {'C','L','O','S','E','-'};
const uint8_t g_aDispOpen[COM_NUM] = {'-','O','P','E','N','-'};
const uint8_t g_aDispError1[COM_NUM] = {'E','r','r','o','r','1'};
const uint8_t g_aDispError2[COM_NUM] = {'E','r','r','o','r','2'};
const uint8_t g_aDispError3[COM_NUM] = {'E','r','r','o','r','3'};


/******************************************************************************
* Function Name: Number_Key_Process
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Number_Key_Process(void)
{
    /* Locate the KEY by key value */
    Valid_Key_Process();
    switch(g_KeyValue)
    {
        case 1:
            g_aInputNumber[g_KeyInputCount] = 4;
            break;
        case 2:
            g_aInputNumber[g_KeyInputCount] = 8;
            break;
        case 4:
            g_aInputNumber[g_KeyInputCount] = 6;
            break;
        case 5:
            g_aInputNumber[g_KeyInputCount] = 2;
            break;
        case 6:
            g_aInputNumber[g_KeyInputCount] = 1;
            break;
        case 7:
            g_aInputNumber[g_KeyInputCount] = 3;
            break;
        case 8:
            g_aInputNumber[g_KeyInputCount] = 7;
            break;
        case 9:
            g_aInputNumber[g_KeyInputCount] = 0;
            break;
        case 10:
            g_aInputNumber[g_KeyInputCount] = 9;
            break;
        case 12:
            g_aInputNumber[g_KeyInputCount] = 5;
            break;  
        default:
            GoTo_ErrorKeyMode();
        
    }
    g_KeyInputCount++;
}   
/******************************************************************************
End of Number_Key_Process function
******************************************************************************/

/******************************************************************************
* Function Name: Valid_Key_Process
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Valid_Key_Process(void)
{
    /* Process the valid key */
    g_fKeyProcessReq = 0;
    g_fKeyOffReq = 1;
    Beep_Times_Set(1);
}
/******************************************************************************
End of Valid_Key_Process function
******************************************************************************/

/******************************************************************************
* Function Name: Invalid_Key_Process
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Invalid_Key_Process(void)
{
    /* Process the invalid key */
    g_fKeyProcessReq = 0;
    g_fKeyOffReq = 1;
}
/******************************************************************************
End of Invalid_Key_Process function
******************************************************************************/

/******************************************************************************
* Function Name: Error_Key_Process
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Error_Key_Process(void)
{
    /* Process the error key */
    g_fKeyProcessReq = 0;
    g_fKeyOffReq = 1;
    Beep_Times_Set(5);
}
/******************************************************************************
End of Error_Key_Process function
******************************************************************************/

/******************************************************************************
* Function Name: Input_Number_Clear
* Description : This function implements Input_Number_Clear function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Input_Number_Clear(void)
{
    /* Process the [clear] key */
    uint8_t i;
    for(i = 0; i < COM_NUM; i++)
    {
        g_aInputNumber[i] = 0xff;
    }
    g_KeyInputCount = 0;
}
/******************************************************************************
End of Input_Number_Clear function
******************************************************************************/

/******************************************************************************
* Function Name: Save_Password_Check
* Description : This function implements Save_Password_Check function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Save_Password_Check(uint8_t ObjectString[])
{
    /* Judgement the password is changed or not? If it is changed, we must set the password save request flag */
    uint8_t i;
    uint8_t result = 0;
    for(i = 0; i < COM_NUM; i++)
    {
        if(ObjectString[i]!= g_aInputNumber[i])
        {
            result = 1;
            break;
        }
    }
    if(1 == result)
    {
        for(i = 0; i < COM_NUM; i++)
        {
            ObjectString[i]= g_aInputNumber[i];
            
        }
        g_fPassWordSaveReq = TRUE;
    }
}
/******************************************************************************
End of Save_Password_Check function
******************************************************************************/

/******************************************************************************
* Function Name: Password_Correct_Check
* Description : This function implements Password_Correct_Check function.
* Arguments : none
* Return Value : none
******************************************************************************/
uint8_t Password_Correct_Check(void)
{
    /* Check the password is correct or not? */
    uint8_t i;
    uint8_t result = 0;
    for(i = 0; i < COM_NUM; i++)
    {
        if(g_aInputNumber[i] != g_aSuperPassWord[i])
        {
            result++;
            break;
        }
    }
    for(i = 0; i < COM_NUM; i++)
    {
        if(g_aInputNumber[i] != g_aPassWord[i])
        {
            result ++;
            break;
        }
    }
    if(2 == result)
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}
/******************************************************************************
End of Password_Correct_Check function
******************************************************************************/

/******************************************************************************
* Function Name: Super_Password_Correct_Check
* Description : This function implements Password_Correct_Check function.
* Arguments : none
* Return Value : none
******************************************************************************/
uint8_t Super_Password_Correct_Check(void)
{
    /* Check the super password is correct or not? */
    uint8_t i;
    for(i = 0; i < COM_NUM; i++)
    {
        if(g_aInputNumber[i] != g_aSuperPassWord[i])
        {
            return FALSE;
        }
    }
    return TRUE;
}
/******************************************************************************
End of Super_Password_Correct_Check function
******************************************************************************/


/******************************************************************************
* Function Name: New_Password_Save
* Description : This function implements Password_Correct_Check function.
* Arguments : none
* Return Value : none
******************************************************************************/
void New_Password_Save(void)
{
    /* Save the new password */
    uint8_t i;
    for(i = 0; i < COM_NUM; i++)
    {
        g_aNewPassWord[i] = g_aInputNumber[i];
    }
}
/******************************************************************************
End of New_Password_Save function
******************************************************************************/

/******************************************************************************
* Function Name: Input_Number_Clear
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Mode_TimeOut_Set(uint16_t Time)
{
    /* Mode timeout setting */
    DI();
    
    ITMCH &= 0x00;                                      /* 12-bit interval timer stop*/
    
    g_ModeTimeOutCount = Time;
    
    ITMCH = (uint8_t)(INTV_TIMER_VALUE_250MS >> 8);     /* 250ms timer */
        ITMCL = (uint8_t)INTV_TIMER_VALUE_250MS;
    
    ITIF = 0;                                           /* Clear INTIT interrupt flag */

        ITMCH |= 0x80;                                  /* 12-bit interval timer start*/
    EI();
}
/******************************************************************************
End of Input_Number_Clear function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_InitalMode
* Description : This function implements GoTo_InitalMode function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_InitalMode(void)
{
    /* Go to the Inital mode */
    Display_Data_Set(g_aDispGood,DISP_INTV_TIMER_1000MS*2);
    g_MainMode = INITIAL_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_InitalMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_IdleMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_IdleMode(void)
{
    /* Go to the idle mode */
    Input_Number_Clear();
    Display_Data_Clear();
    g_MainMode = IDLE_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_IdleMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_ClearKeyWaitMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_ClearKeyWaitMode(void)
{
    /* Go to the [clear] key wait mode */
    Mode_TimeOut_Set(CLEAR_WAIT_MODE_TIMEOUT_VALUE);
    g_MainMode = WAIT_CLEARKEY_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_ClearKeyWaitMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_InputMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_InputMode(void)
{
    /* Go to the password input mode */
    Mode_TimeOut_Set(INPUT_MODE_TIMEOUT_VALUE);
    Display_Input_Numbers();
    g_MainMode = INPUT_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_IdleMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_ErrorKeyMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_ErrorKeyMode(void)
{
    /* Go to the error key pressed mode */
    Display_Data_Set(g_aDispError,DISP_INTV_TIMER_1000MS*2);
    g_MainMode = ERROR_KEY_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_ErrorKeyMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_EepromErrorMode
* Description : This function implements GoTo_EepromErrorMode function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_EepromErrorMode(void)
{
    /* Go to the EEPROM accessed error mode */
    Display_Data_Set(g_aDispError3,DISP_INTV_TIMER_1000MS*15);
    g_MainMode = EEPROM_ERROR_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_EepromErrorMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_HoldMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_HoldMode(void)
{
    /* Go to the hold mode */
    Mode_TimeOut_Set(HOLD_MODE_TIMEOUT_VALUE);
    Display_Data_Set(g_aDispHold15,DISP_INTV_TIMER_1000MS*15);
    g_PasswordInputErrorCount = 0;
    g_MainMode = HOLD_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_HoldMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_PasswordModifyMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_PasswordModifyMode(void)
{
    /* Go to the super password modify mode */
    Input_Number_Clear();
    Display_Data_Set(g_aDispProg,DISP_INTV_TIMER_1000MS*2);
    g_MainMode = PASSWORD_MODIFY_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_PasswordModifyMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_DoorOpeningMode
* Description : This function implements key process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_DoorOpeningMode(void)
{
    /* Go to the door opening mode */
    Display_Input_Numbers();
    g_DispTimeCount = DISP_INTV_TIMER_1000MS/4;
    g_PasswordInputErrorCount = 0;
    g_MainMode = DOOR_OPENING_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_DoorOpeningMode function
******************************************************************************/

/******************************************************************************
* Function Name: GoTo_DoorClosingMode
* Description : This function implements GoTo_DoorClosingMode function.
* Arguments : none
* Return Value : none
******************************************************************************/
void GoTo_DoorClosingMode(void)
{
    /* Go to the door closing mode */
    Display_Input_Numbers();
    g_DispTimeCount = DISP_INTV_TIMER_1000MS/4;
    g_MainMode = DOOR_CLOSING_MODE;
    g_SubMode = 0;
}
/******************************************************************************
End of GoTo_DoorClosingMode function
******************************************************************************/

/******************************************************************************
* Function Name: Mode_Process
* Description : This function implements Mode_Process function.
* Arguments : none
* Return Value : none
******************************************************************************/
void Mode_Process(void)
{
    /* Mode process */
    switch(g_MainMode)
    {
        case INITIAL_MODE:
        case EEPROM_ERROR_MODE:
        case ERROR_KEY_MODE:
            /* Initial mode, EEPROM access error mode and error key pressed mode process */
            if(1 == g_fKeyProcessReq)
            {
                Invalid_Key_Process();
            }
            if(g_DispTimeCount <= 0)
            {
                GoTo_IdleMode();
            }
            break;
        case IDLE_MODE:
            /* Idle mode process */
            if(1 == g_fKeyProcessReq)
            {
                switch(g_KeyValue)
                {
                    case 3:         /* [lock] key was pressed */
                        if(TRUE == g_fDoorOpen)
                        {
                            /* door was opened */
                            Error_Key_Process();
                            GoTo_ErrorKeyMode();
                        }
                        else
                        {
                            /* door was closed */
                            Valid_Key_Process();
                        }
                        break;
                    case 11:        /* [clear] key was pressed */
                        if(TRUE == g_fDoorOpen) 
                        {
                            /* door was opened */
                            Invalid_Key_Process();
                            GoTo_ClearKeyWaitMode();
                        }
                        else
                        {
                            /* door was closed */
                            Valid_Key_Process();
                        }
                        break;
                    default:        /* Number key*/
                        Number_Key_Process();
                        GoTo_InputMode();
                    
                }
            }
            break;
        case WAIT_CLEARKEY_MODE:
            /* [Clear] key wait mode process */
            if(g_ModeTimeOutCount>0)
            {
                if(1 == g_fKeyProcessReq)
                {
                    switch(g_KeyValue)
                    {
                        case 3:         /* [lock] key was pressed */
                            Error_Key_Process();
                            GoTo_ErrorKeyMode();
                            break;
                        case 11:        /* [clear] key was pressed */
                            Valid_Key_Process();
                            GoTo_PasswordModifyMode();
                            break;
                        default:
                            Number_Key_Process();
                            GoTo_InputMode();       
                    }
                }
            }
            else
            {   /*[Clear] key wait timeout*/
                if(1 == g_fKeyProcessReq)
                {
                    Invalid_Key_Process();
                }
                GoTo_IdleMode();
            }
            break;
        case INPUT_MODE:
            /* password input mode process */
            if(g_ModeTimeOutCount>0)
            {
                if(1 == g_fKeyProcessReq)
                {
                    switch(g_KeyValue)
                    {
                        case 3:         /* [lock] key was pressed */
                            if(TRUE == g_fDoorOpen)
                            {
                                /* door was opened */
                                if(g_KeyInputCount>=COM_NUM)
                                {
                                    Save_Password_Check(g_aPassWord);
                                    Valid_Key_Process();
                                    GoTo_DoorClosingMode();
                                }
                                else
                                {
                                    Error_Key_Process();
                                    GoTo_ErrorKeyMode();
                                }
                            }
                            else
                            { 
                                /* door was closed */
                                Valid_Key_Process();
                                GoTo_IdleMode();
                            }
                            break;
                        case 11:        /* [clear] key was pressed */
                            Valid_Key_Process();
                            GoTo_IdleMode();
                            break;
                        default:        /* [Number] key was pressed */
                            Number_Key_Process();
                            Display_Input_Numbers();
                            g_ModeTimeOutCount = INPUT_MODE_TIMEOUT_VALUE;
                            if(TRUE != g_fDoorOpen)
                            {
                                if(g_KeyInputCount>=COM_NUM)
                                {
                                    if(Password_Correct_Check() == TRUE)
                                    {
                                        GoTo_DoorOpeningMode();
                                    }
                                    else
                                    {
                                        g_PasswordInputErrorCount++;
                                        if(g_PasswordInputErrorCount>=PERMIT_ERROR_TIME)
                                        {
                                            Error_Key_Process();
                                            GoTo_HoldMode();
                                        }
                                        else
                                        {
                                            Error_Key_Process();
                                            GoTo_ErrorKeyMode();
                                        }
                                    }
                                }
                            }
                            else
                            {
                                if(g_KeyInputCount>COM_NUM)
                                {
                                    Error_Key_Process();
                                    GoTo_ErrorKeyMode();
                                }
                            }
                    }
                }
            }
            else
            {   /*Number key wait timeout*/
                if(1 == g_fKeyProcessReq)
                {
                    Invalid_Key_Process();
                }
                GoTo_IdleMode();
            }
            break;
        case PASSWORD_MODIFY_MODE:
            /* Super password modify mode process */
            switch (g_SubMode)
            {
                case 0: 
                case 3:
                case 6:
                    if(1 == g_fKeyProcessReq)
                    {
                        Invalid_Key_Process();
                        
                    }
                    if(g_DispTimeCount <= 0)
                    {
                        Mode_TimeOut_Set(PASSWORD_MODIFY_TIMEOUT_VALUE);
                        Input_Number_Clear();
                        Display_Data_Clear();
                        g_SubMode++;
                    }
                    break;
                
                case 1:
                case 4:
                    if(g_ModeTimeOutCount>0)
                    {
                        if(1 == g_fKeyProcessReq)
                        {
                            switch(g_KeyValue)
                            {
                                case 3:         /* [lock] key was pressed */
                                    Error_Key_Process();
                                    GoTo_ErrorKeyMode();
                                    break;
                                case 11:        /* [clear] key was pressed */
                                    Valid_Key_Process();
                                    GoTo_IdleMode();
                                    break;
                                default:        /* [Number] key was pressed */
                                    Number_Key_Process();
                                    Display_Input_Numbers();
                                    if(g_KeyInputCount>=COM_NUM)
                                    {
                                        if(g_SubMode == 1)
                                        {
                                            if(Super_Password_Correct_Check() == TRUE)
                                            {
                                                Display_Input_Numbers();
                                                g_DispTimeCount = DISP_INTV_TIMER_1000MS/4;
                                                g_SubMode++;
                                            }
                                            else
                                            {
                                                Error_Key_Process();
                                                GoTo_ErrorKeyMode();
                                            }
                                        }
                                        else
                                        {
                                            New_Password_Save();
                                            Display_Input_Numbers();
                                            g_DispTimeCount = DISP_INTV_TIMER_1000MS/4;
                                            g_SubMode++;
                                            
                                        }
    
                                    }               
                                    else
                                    {
                                        g_ModeTimeOutCount = INPUT_MODE_TIMEOUT_VALUE;
                                    }
                            }
                        }
                    }   
                    else
                    {   
                        /*Number key wait timeout*/
                        if(1 == g_fKeyProcessReq)
                        {
                            Invalid_Key_Process();
                        }
                        GoTo_IdleMode();
                    }
                    break;
                case 2:
                case 5:
                case 8:
                    if(1 == g_fKeyProcessReq)
                    {
                        Invalid_Key_Process();
                        
                    }
                    if(g_DispTimeCount <= 0)
                    {
                        if(2 == g_SubMode)
                        {
                            Display_Data_Set(g_aDispNew,DISP_INTV_TIMER_1000MS*2);
                        }
                        else if(5 == g_SubMode)
                        {
                            Display_Data_Set(g_aDispAgain,DISP_INTV_TIMER_1000MS*2);
                        }
                        else
                        {
                            Display_Data_Set(g_aDispDone,DISP_INTV_TIMER_1000MS*2);
                        }
                        g_SubMode++;
                    }
                    break;
                case 7:
                    if(g_ModeTimeOutCount>0)
                    {
                        if(1 == g_fKeyProcessReq)
                        {
                            switch(g_KeyValue)
                            {
                                case 3:         /* [lock] key was pressed */
                                case 11:        /* [clear] key was pressed */
                                    Error_Key_Process();
                                    GoTo_ErrorKeyMode();
                                    break;
                                default:        /* [Number] key was pressed */
                                    Number_Key_Process();
                                    Display_Input_Numbers();
                                    g_ModeTimeOutCount = INPUT_MODE_TIMEOUT_VALUE;
                                    if(g_aNewPassWord[g_KeyInputCount-1] != g_aInputNumber[g_KeyInputCount-1])
                                    {
                                        Error_Key_Process();
                                        GoTo_ErrorKeyMode();
                                    }
                                    else
                                    {
                                        if((g_KeyInputCount>=COM_NUM)&&(g_aNewPassWord[COM_5] == g_aInputNumber[COM_5]))
                                        {
                                        
                                            Save_Password_Check(g_aSuperPassWord);
                                            Display_Input_Numbers();
                                            g_DispTimeCount = DISP_INTV_TIMER_1000MS/4;
                                            g_SubMode++;
                                        }               
                                        
                                    }
                            }
                        }
                    }   
                    else
                    {   /*Number key wait timeout*/
                        if(1 == g_fKeyProcessReq)
                        {
                            Invalid_Key_Process();
                        }
                        GoTo_IdleMode();
                    }
                    break;
                case 9:
                    if(1 == g_fKeyProcessReq)
                    {
                        Invalid_Key_Process();
                    }
                    if(g_DispTimeCount <= 0)
                    {
                        GoTo_IdleMode();
                    }   
            }
            break;
        case DOOR_OPENING_MODE:
            /* Door opening mode process */
            if(1 == g_fKeyProcessReq)
            {
                Invalid_Key_Process();
                        
            }
            switch (g_SubMode)
            {
                case 0:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispOpen,DISP_INTV_TIMER_1000MS/2,COM_0);
                        g_MotorWorkTimeCount = MOTOR_CYCLE_TIME;
                        Motor_On();
                        g_SubMode++;
                    }
                    break;
                case 1:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispOpen,DISP_INTV_TIMER_1000MS/2,COM_1);
                        g_SubMode++;
                    }
                    break;
                case 2:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispOpen,DISP_INTV_TIMER_1000MS/2,COM_2);
                        g_SubMode++;
                    }
                    break;  
                case 3:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispOpen,DISP_INTV_TIMER_1000MS/2,COM_3);
                        g_SubMode++;
                    }
                    break;  
                case 4:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispOpen,DISP_INTV_TIMER_1000MS/2,COM_4);
                        g_SubMode++;
                    }
                    break;  
                case 5:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispOpen,DISP_INTV_TIMER_1000MS/2,COM_5);
                        g_SubMode++;
                    }
                    break;
                case 6:
                    if(g_DispTimeCount <= 0)
                    {
                        if(TRUE == g_fDoorOpen)
                        {
                            /* Door Open successed */
                            Display_Data_Set(g_aDispOpened,DISP_INTV_TIMER_1000MS*2);
                        } 
                        else
                        {   /* Door Open failed */
                            Display_Data_Set(g_aDispError1,DISP_INTV_TIMER_1000MS*5);
                            Beep_Times_Set(5);
                            
                        }
                        g_SubMode++;
                    }
                    break;  
                case 7:
                    if(g_DispTimeCount <= 0)
                    {
                        /* Display timeout */
                        GoTo_IdleMode();
                    }
            }
            break;
        case DOOR_CLOSING_MODE:
            /* Door closing mode process */
            if(1 == g_fKeyProcessReq)
            {
                Invalid_Key_Process();
                        
            }
            switch (g_SubMode)
            {
                case 0:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispClose,DISP_INTV_TIMER_1000MS/2,COM_0);
                        if(0 == g_DoorOpenTime)
                        {
                            g_MotorWorkTimeCount = DOOR_CLOSE_TIME;
                        }
                        else
                        {
                            g_MotorWorkTimeCount = MOTOR_CYCLE_TIME - g_DoorOpenTime;
                            g_DoorOpenTime = 0;
                            if(g_MotorWorkTimeCount < DOOR_CLOSE_TIME)
                            {
                                g_MotorWorkTimeCount = DOOR_CLOSE_TIME;
                            }
                        }
                        Motor_On();
                        g_SubMode++;
                    }
                    break;
                case 1:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispClose,DISP_INTV_TIMER_1000MS/2,COM_1);
                        g_SubMode++;
                    }
                    break;
                case 2:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispClose,DISP_INTV_TIMER_1000MS/2,COM_2);
                        g_SubMode++;
                    }
                    break;  
                case 3:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispClose,DISP_INTV_TIMER_1000MS/2,COM_3);
                        g_SubMode++;
                    }
                    break;  
                case 4:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispClose,DISP_INTV_TIMER_1000MS/2,COM_4);
                        g_SubMode++;
                    }
                    break;  
                case 5:
                    if(g_DispTimeCount <= 0)
                    {
                        Display_Dynamic_Data_Set(g_aDispClose,DISP_INTV_TIMER_1000MS/2,COM_5);
                        g_SubMode++;
                    }
                    break;
                case 6:
                    if(g_DispTimeCount <= 0)
                    {
                        if(TRUE != g_fDoorOpen)
                        {
                            /* Door close successed */
                            Display_Data_Set(g_aDispClosed,DISP_INTV_TIMER_1000MS*2);
                        } 
                        else
                        {
                            /* Door close failed */
                            Display_Data_Set(g_aDispError2,DISP_INTV_TIMER_1000MS*5);
                            Beep_Times_Set(5);
                            
                        }
                        g_SubMode++;
                    }
                    break;  
                case 7:
                    if(g_DispTimeCount <= 0)
                    {
                        /* Display timeout */
                        GoTo_IdleMode();
                    }
            }
            break;
        case HOLD_MODE: 
            /* Hold mode process */
            if(1 == g_fKeyProcessReq)   
            {
                /* Key was pressed */
                Invalid_Key_Process();
                Display_Data_Set(g_aDispHold,DISP_INTV_TIMER_1000MS*2);
            }
            if(g_DispTimeCount <= 0)
            {
                /* Clear display */
                Display_Data_Clear();
            }
            if(g_ModeTimeOutCount<=0)
            {
                /* Hold mode timeout */
                GoTo_IdleMode();
            }
            break;
    }
                
}
/******************************************************************************
End of Mode_Process function
******************************************************************************/

