/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : monitor.c
* Version : 1.0
* Device(s) : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description : This file implements device driver for sensor.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.09.2014 1.00 First Release
******************************************************************************/

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "monitor.h"
#include "beep.h"
#include "e2prom.h"
#include "timer.h"
#include "adc.h"

/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
uint8_t g_SensorSimpleTimes;			/* Record simple times of dust sensor */
uint8_t g_fSensorComputeReq;			/* Dust sensor computes request flag */		
uint8_t g_fSensorSimpling;			/* Flag indicates dust sensor is simpling */
uint8_t g_fSensorStartReq;			/* Dust sensor starts request flag */
uint8_t g_fSensorStopReq; 			/* Dust sensor stops request flag */
uint8_t g_fCheckAlarmDataReq; 			/* Dust density data check request flag */
uint16_t g_aSensorAdValue[SENSOR_SIMPLE_TIMES];	/* Record dust sensor simple value */
uint16_t g_DustDensityValue;			/* Record dust density value */
uint16_t g_DustDensityMinValue;			/* Record dust density min value */
uint16_t g_fSensorUpdateCounter;		/* Dust density valute update counter */
uint16_t g_fDispFlashCounter;			/* Dust density valute flash counter */
_DUST_ALARM_DATA g_sDustAlarmData;		/* Record the alarm value of dust density */
_DUST_ALARM_DATA g_sDustAlarmTempData;		/* Record the temporary alarm value of dust density */
uint8_t g_DustAlarmDataSetStep;			/* Record the step of alarm value setting */
uint8_t g_fAlarmDataSetFlashOn;			/* Alarm data setting flash on request flag */
uint8_t g_fAlarmOnFlashOn;			/* Alarm on flag flash on request flag */

extern uint8_t g_MainMode;
extern uint8_t g_aDataBuffer[];
extern uint8_t g_fNewDataDispRq;
extern uint16_t g_ModeTimeOutCounter;

/***********************************************************************************************************************
* Function Name: Get_Sensor_Dust_Density
* Description  : This function gets the density of PM2.5(V(get)=57*10/100*density+V(min),V(get)=DAv(get)*5000/1024)
		->density = (DAv(get)-DAv(min))*86/100 (density unit: ug/m^3)
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Get_Dust_Density(void)
{
	uint16_t i,j,min,tmp;
	uint16_t average = 0;
	
	/* Sort the array */
	for(i=0;i<SENSOR_SIMPLE_TIMES;i++)
	{
		min = g_aSensorAdValue[i];
		for(j=i;j<SENSOR_SIMPLE_TIMES;j++)
		{
			if(g_aSensorAdValue[j]<min)
			{
				tmp = min;
				min = g_aSensorAdValue[j];
				g_aSensorAdValue[j] = tmp;
			}
		}
			g_aSensorAdValue[i] = min;
		}
		/*Get average value*/
		tmp = 0;
		for(i=0;i<SENSOR_SIMPLE_TIMES-2;i++)
		{
			tmp += g_aSensorAdValue[i+1];
		}
		average = (tmp/(SENSOR_SIMPLE_TIMES-2)) + (tmp%(SENSOR_SIMPLE_TIMES-2)/((SENSOR_SIMPLE_TIMES-2)/2));
		
		/*Get air quality index value*/
		if(average>=g_DustDensityMinValue )
		{
			tmp = average - g_DustDensityMinValue;
			if(AD_SENSOR_MAX_OFFSET_VALUE<=tmp)
			{
				/* density of PM2.5 = Max Vaule = 500 */
				g_DustDensityValue = 500;
			}
			else
			{ 
				/* density of PM2.5 = (DAv(get)-DAv(min))*86/100 */
				tmp = tmp*86;
				g_DustDensityValue = (tmp/100)+((tmp%100)/50);
			}
		}
		else
		{
			//save average as new value of g_DustDensityMinValue
			g_DustDensityMinValue = average;
			g_DustDensityValue = 0;
			*(uint16_t *)&g_aDataBuffer[0] = g_DustDensityMinValue;
		 	EEPROM_WritePage(0x0);	
		}
		
}
/***********************************************************************************************************************
End of function Get_Dust_Density
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: Check_Alarm_Data
* Description  : This function checks the alarm data 
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Check_Alarm_Data(void)
{
	uint16_t DustDensityAlarmValue;
	DustDensityAlarmValue = g_sDustAlarmData.High*100 +  g_sDustAlarmData.Middle*10 +  g_sDustAlarmData.Low;
	if(g_DustDensityValue >= DustDensityAlarmValue)
	{
		GoTo_AlarmOnMode();
	}	
}
/***********************************************************************************************************************
End of function Check_Alarm_Data
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: Sensor_Stop
* Description  : This function stops the sensor
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Sensor_Stop(void)
{
	DI();
	TT0 |= 0x0007;                /* Stops TAU00~TAU02 */
	ADC_Stop();
	g_SensorSimpleTimes = 0;
	g_fSensorComputeReq = 0;
	g_fSensorSimpling = OFF;
	EI();
}
/***********************************************************************************************************************
End of function Sensor_Stop
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: Sensor_Start
* Description  : This function starts the sensor
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void Sensor_Start(void)
{
	DI();
	g_SensorSimpleTimes = 0;
	g_fSensorComputeReq = 0;
	TAU0_PWM_Out_Start();
	g_fSensorSimpling = ON;
	EI();
}
/***********************************************************************************************************************
End of function Sensor_Start
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: GoTo_NormalMode
* Description  : This function does some preparative works before goto normal mode
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void GoTo_NormalMode(void)
{
	if((g_MainMode == INITIAL_MODE)||(g_MainMode == BATTERY_MODE)||(g_MainMode == LOW_POWER_MODE))
	{
		g_fSensorStartReq = ON;
	}
	if(g_MainMode == ALARM_ON_MODE)
	{
		Alarm_Off();
	}
	BATTERY_EN = OFF;
	BATTEEY_LED = OFF;
	g_fAlarmOnFlashOn = OFF;
	g_MainMode = NORMAL_MODE;
	g_fNewDataDispRq = ON;
	RTC_Stop();
}
/***********************************************************************************************************************
End of function GoTo_NormalMode
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: GoTo_AlarmSettingMode
* Description  : This function does some preparative works before goto alarm setting mode
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void GoTo_AlarmSettingMode(void)
{
	g_sDustAlarmTempData.High = g_sDustAlarmData.High;
	g_sDustAlarmTempData.Middle = g_sDustAlarmData.Middle;
	g_sDustAlarmTempData.Low = g_sDustAlarmData.Low;
	g_ModeTimeOutCounter = ALARM_SETTING_TIMEOUT_TIME;
	g_fDispFlashCounter = SENSOR_DISP_FLASH_TIME;
	g_fAlarmDataSetFlashOn = ON;
	g_fAlarmOnFlashOn = OFF;
	g_MainMode = ALARM_SETTING_MODE;
	g_DustAlarmDataSetStep = 0;
	g_fNewDataDispRq = ON;
}
/***********************************************************************************************************************
End of function GoTo_AlarmSettingMode
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: GoTo_AlarmOnMode
* Description  : This function does some preparative works before goto alarm on mode
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void GoTo_AlarmOnMode(void)
{
	g_fDispFlashCounter = SENSOR_DISP_FLASH_TIME;
	Alarm_On();
	g_fAlarmOnFlashOn = ON;
	g_MainMode = ALARM_ON_MODE;
}
/***********************************************************************************************************************
End of function GoTo_AlarmOnMode
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: GoTo_AlarmStandbyMode
* Description  : This function does some preparative works before goto alarm standby mode
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void GoTo_AlarmStandbyMode(void)
{
	g_fAlarmOnFlashOn = ON;
	g_fCheckAlarmDataReq = ON;
	g_MainMode = ALARM_STANDBY_MODE;
}
/***********************************************************************************************************************
End of function GoTo_AlarmStandbyMode
***********************************************************************************************************************/

/******************************************************************************
* Function Name : Sensor_Process
* Description   : This function implements sensor process.
* Arguments     : density 
* Return Value  : none
******************************************************************************/
void Sensor_Process(void)
{
	if((g_MainMode==NORMAL_MODE)||(g_MainMode==ALARM_SETTING_MODE)||(g_MainMode==ALARM_STANDBY_MODE)||(g_MainMode==ALARM_ON_MODE))
	{
		
		if(g_fSensorUpdateCounter>0)
		{
			g_fSensorUpdateCounter--;
			
		}
		else
		{
			g_fSensorStartReq = 1;
		}
		if(1U == g_fSensorComputeReq)
		{
			g_fSensorComputeReq = 0;
			Get_Dust_Density();
			if(g_MainMode!=ALARM_SETTING_MODE)
			{
				g_fNewDataDispRq = ON;	
			}
			if(g_MainMode==ALARM_STANDBY_MODE)
			{
				g_fCheckAlarmDataReq = ON;
			}
		}
	}
	if(1U == g_fSensorStartReq)
	{
		g_fSensorStartReq = 0;
		if(g_fSensorSimpling == OFF)
		{
			g_fSensorUpdateCounter = SENSOR_UPDATE_MAX_TIME;
			Sensor_Start();
		}
	}
	if(1U == g_fSensorStopReq)
	{
		g_fSensorStopReq = 0;
		g_fSensorUpdateCounter = 0;
		Sensor_Stop();
	} 
}
/******************************************************************************
End of function Sensor_Process
******************************************************************************/