/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : system.c
* Version : 1.0
* Device(s) : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description : This file implements device driver for initializes system.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
* : 20.09.2015 1.00 First Release
******************************************************************************/


/*******************************************************************************
** Pragma directive
********************************************************************************/
#pragma interrupt INTP0 Interrupt_Intp0
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "system.h"
#include "lcd.h"
#include "timer.h"
#include "adc.h"
#include "battery.h"
#include "monitor.h"
/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/

extern uint8_t g_MainMode;
extern uint8_t g_fSwitchMonitorOnReq;
extern uint8_t g_fSleepModeOn;

static void CLOCK_Init(void);
static void PORT_Init(void);
static void INTP0_Init(void);

/******************************************************************************
* Function Name : CLOCK_Init
* Description : This function initialize clock setting.
* Arguments : none
* Return Value : none
******************************************************************************/
static void CLOCK_Init(void)
{
	uint8_t w_count;
    	/* Set fMX */
	CMC = 0x12;		/* Selects XT1 */
	MSTOP = 1;         	/* Stops X1 */
	/* Set fMAIN */
	MCM0 = 0;		/* Selects fIH as the main system clock */
    	/* Set fSUB */
    	XTSTOP = 0;       	/* Starts XT1 */

    	/* Change the waiting time according to the system */
    	for (w_count = 0; w_count <= 30; w_count++)
    	{
        	NOP();
    	}

    	OSMC = 0x00;      /* Selects fSUB */      
}
/******************************************************************************
End of function CLOCK_Init
******************************************************************************/
/******************************************************************************
* Function Name : PORT_Init
* Description : This function sets used ports.
* Arguments : none
* Return Value : none
******************************************************************************/
static void PORT_Init(void)
{
	/* Set Fn and Bell keys */
    	PFSEG2 &= 0xcf;		/* PFDEG21,PFDEG20 = 0 */
	PM6.0 = 1;		/* Bell key */
    	PM6.1 = 1;		/* Fn key */
	
    	/* Set switch */
    	//PM13.7 = 1;           /* input mode */
	
    	/* Set EEPROM communication port */
    	PFSEG4 &= 0x73;		/* PFDEG35,PFDEG34 = 0 */
	PMC14 &= 0xe7;		/* PMC144,PMC143 = 0 */
    	PM14.3 = 0;		/* SDA */
    	PM14.4 = 0;		/* SCL */
	
	/* Set charger IC stautes indicator and control port */
	PFSEG3 &= 0x7f;		/* PFDEG31 = 0 */
	PFSEG4 &= 0x7e;		/* PFDEG32 = 0 */
	PMC1 &= 0xe7;		/* PMC14,PMC13 = 0 */
    	PM1.3 = 1;		/* STDBY */
    	PM1.4 = 0;		/* charge control */
	
	/* Set discharger IC enable/disable control port */
	PFSEG3 &= 0xbf;		/* PFDEG30 = 0 */
	PM1.2 = 0;		/* U5_EN */
	P1.2 = 0;		/* U5_EN disable */
	
	/* Set buzzer output control port */
	PFSEG3 &= 0xdf;		/* PFDEG29 = 0 */
	PM1.1 = 0;		/* Buzzer output */
	P1.1 = 0;		/* Buzzer off */
	
	/* Set PWM output port */
	PFSEG2 &= 0xf7;		/* PFDEG19 = 0 */
	PM3.0 = 0;		/* TO01 */
	P3.0 = 0;	
	
	/* Set ADC input port */
	ADPC = 0x03;
	PM2.1 = 1;
	PM2.0 = 1;
	PM14.2 = 1;
	
	/* Discharge indicator LED */
	PFSEG3 &= 0xfb;		/* PFDEG26 = 0 */
	PM14.1 = 0;
}
/******************************************************************************
End of function PORT_Init
******************************************************************************/

/******************************************************************************
* Function Name : INTP0_Init
* Description : This function initializes INTP0 setting.
* Arguments : none
* Return Value : none
 ******************************************************************************/
static void INTP0_Init(void)
{
    PMK0 = 1;               /* Mask intp0 interrupt */      
    PIF0 = 0;               /* Clear interrupt flag */
    
    PPR00 = 0;
    PPR10 = 0;              /* Specify intp0 int level(0=high priority level) */
    
    EGP0.0 = 1;             /* Both rising and falling edges */
    EGN0.0 = 1;
    
    PMK0 = 0;               /* Enable INTP0 interrupt */
}
/******************************************************************************
End of function INTP0_Init
******************************************************************************/

/******************************************************************************
* Function Name : System_Init
* Description : This function initializes system .
* Arguments : none
* Return Value : none
******************************************************************************/
void System_Init(void)
{
	DI();
	/* Clock initialize */
    	CLOCK_Init();
    
    	/* Port initialize */
    	PORT_Init();        

    	/* Initialize TAU0 module */
    	TAU0_Init();     
	
	/* Initialize RTC module */
	RTC_Init();
    
    	/* Initialize INTP0 */
    	INTP0_Init();   
	
	/* Initialize LCD */
	LCD_Init();
	
	/* Initialize ADC */
	ADC_Init();
	
	EI();
}
/******************************************************************************
End of function System_Init
******************************************************************************/

/******************************************************************************
* Function Name: Interrupt_Intp0
* Description : This function is INTP0 interrupt service routine.
* Arguments : none
* Return Value : none
******************************************************************************/
__interrupt void Interrupt_Intp0(void)
{
	if(g_MainMode != LOW_POWER_MODE)
	{
		if(1U == g_fSleepModeOn)
		{
			g_fSleepModeOn = 0U;
			LCD_Start();
		}
		if(0U == g_fSwitchMonitorOnReq)
		{
			if(1U == P13.7)	
			{	
				g_fSwitchMonitorOnReq = 1;
				GoTo_NormalMode();			
			}
		}
		else
		{	
			if(0U == P13.7)	
			{
				g_fSwitchMonitorOnReq = 0;		
				GoTo_BatteryMode();
			}
		}
	}
}
/******************************************************************************
End of Interrupt_Intp0 function 
******************************************************************************/