/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name    : timer.c
* Version      : 1.0
* Device(s)    : R7F0C001
* H/W Platform : YCB15R7F0C001001B
* Description  : This file implements device driver for timer.
******************************************************************************/
/******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 20.09.2015 1.00 First Release
******************************************************************************/


/*******************************************************************************
** Pragma directive
*******************************************************************************/
#pragma interrupt INTRTC INTRTC_interrupt
#pragma interrupt INTTM01 INTTM01_Interrupt
#pragma interrupt INTTM02 INTTM02_Interrupt
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/******************************************************************************
Includes <System Includes> , <Project Includes>
******************************************************************************/
#include "userdefine.h"
#include "timer.h"
#include "adc.h"
/******************************************************************************
Macro definitions
******************************************************************************/


/******************************************************************************
Exported global variables and functions(can be accessed by other files)
******************************************************************************/
extern uint8_t g_MainMode;
extern uint8_t g_fLowPowerSleepOn;
extern uint8_t g_fNewDataDispRq;
/******************************************************************************
* Function Name : TAU0_Init
* Description   : This function initializes the TAU0 module.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void TAU0_Init(void)
{
	TAU0EN = 1;		/* Clock supply is started and writing to each register is enabled */
	TPS0 |= 0x0064;		/* Clock frequencies of CK00 = fCLK/16 = 1.5MHz,CK01 = fCLK/64 = 375KHz */
	TMR00 = 0x8001;		/* Master operation mode:Selects CK01 as operation clock,PWM output (master) mode */									 
	TMR01 = 0x8409;		/* Slave operation mode:Selects CK01 as operation clock (same as master channel),Selects INTTM00 of master channel,PWM output (slave) mode */			
	TMR02 = 0x0000;		/* Selects CK00 as operation clock,Interval timer */
	TMR03 = 0x8000;		/* Selects CK01 as operation clock,Interval timer */
	
	TDR00 = 18749;		/* Master pulse period=1/375KHz*(18749+1)=50ms  */
	TDR01 = 18630;		/* Slave pulse width=1/375KHz*18630=49.68ms */
	TDR02 = 419;		/* Interval timer period=1/1.5MHz*(419+1)=280us */
	TDR03 = 37499;		/* Interval timer period=1/375KHz*(37499+1)=100ms */
	
	TOM0 |= 0x02;		/* Channel1 is slave mode */
	TOL0 |= 0x00;		/* Channel1 is negative logic ("H" is active) */
	TO0 = 0x02;		/* Default level of channel1 "H" */
	TOE0 |= 0x02;		/* Channel1 is output */
	
	TMIF00 = 0;             /* Clear channel0 interrupt flag */
    	TMIF01 = 0;             /* Clear channel1 interrupt flag */
	TMIF02 = 0;             /* Clear channel2 interrupt flag */
	TMIF03 = 0;             /* Clear channel3 interrupt flag */
	
    	TMMK01 = 0;             /* enable channel1 interrupt */
	TMMK02 = 0;             /* enable channel2 interrupt */
}
/******************************************************************************
End of function TAU0_Init
******************************************************************************/

/******************************************************************************
* Function Name : TAU0_PWM_Out_Start
* Description   : This function starts TAU0 channel 0 ~ 1 counter.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void TAU0_PWM_Out_Start(void)
{
    TS0 |= 0x0003;                /* Starts TAU00~TAU01 */
}
/******************************************************************************
End of function TAU0_Channel_Start
******************************************************************************/

/******************************************************************************
* Function Name : TAU0_PWM_Out_Stop
* Description   : This function stops TAU0 channel 0 ~ 1 counter.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void TAU0_PWM_Out_Stop(void)
{
    TT0 |= 0x0003;		/* Stops TAU00~TAU01 */
}
/******************************************************************************
End of function TAU0_PWM_Out_Stop
******************************************************************************/

/******************************************************************************
* Function Name : RTC_Init
* Description   : This function initializes the RTC module.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void RTC_Init(void)
{
    RTCEN = 1;			/* supply RTC clock */
    RTCC0 = 0x02;		/*Once per 1 s (same time as second count up)*/
    RTCMK = 1;			/* disable INTRTC interrupt */
    RTCIF = 0;			/* clear INTRTC interrupt flag */     
}
/******************************************************************************
End of function RTC_Init
******************************************************************************/

/******************************************************************************
* Function Name : RTC_Start
* Description   : This function starts RTC counter.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void RTC_Start(void)
{
    RTCIF = 0;    		/* clear INTRTC interrupt flag */
    RTCMK = 0;    		/* enable INTRTC interrupt */
    RTCE = 1;     		/* Starts counter operation */
}
/******************************************************************************
End of function RTC_Start
******************************************************************************/

/******************************************************************************
* Function Name : RTC_Stop
* Description   : This function stops RTC counter.
* Arguments     : none
* Return Value  : none
******************************************************************************/
void RTC_Stop(void)
{
    RTCIF = 0;    		/* clear INTRTC interrupt flag */
    RTCMK = 1;    		/* disable INTRTC interrupt */
    RTCE = 0;     		/* Stops counter operation */
}
/******************************************************************************
End of function RTC_Stop
******************************************************************************/

/******************************************************************************
* Function Name: INTRTC_interrupt
* Description : This function is INTRTC interrupt service routine.
* Arguments : none
* Return Value : none
******************************************************************************/
__interrupt void INTRTC_interrupt(void)
{
	if(g_MainMode == LOW_POWER_MODE)
	{
		g_fLowPowerSleepOn = 1 - g_fLowPowerSleepOn;
	}
	if(0U == g_fLowPowerSleepOn)
	{
		g_fNewDataDispRq = ON;
	}
} 
/******************************************************************************
End of function INTRTC_interrupt
******************************************************************************/

/******************************************************************************
* Function Name: INTTM01_Interrupt
* Description : This function is INTTM01 interrupt service routine.
* Arguments : none
* Return Value : none
******************************************************************************/
__interrupt void INTTM01_Interrupt(void)
{
	TS0 |= 0x0004;		/* Starts TAU02 */
}
/******************************************************************************
End of function INTTM01_Interrupt
******************************************************************************/

/******************************************************************************
* Function Name: INTTM02_Interrupt
* Description : This function is INTTM02 interrupt service routine.
* Arguments : none
* Return Value : none
******************************************************************************/
__interrupt void INTTM02_Interrupt(void)
{
	TT0 |= 0x0004;		/* Stops TAU02 */
	ADS = SENSOR_ANI_CH;	/* select ANI1 as input pin */
    	ADC_Start();		/* start a/d conversion */
}
/******************************************************************************
End of function INTTM01_Interrupt
******************************************************************************/